@@ -3,25 +3,50 @@ import math
 import os
 import random
 import subprocess
-import warnings
-from typing import Any, Set, Union
+from socket import gethostname
+from typing import Any, Dict, Set, Tuple, Union
 
 import numpy as np
 import torch
 from loguru import logger
 from torch import Tensor
 from torch._six import string_classes
+from torch.autograd import Function
 from torch.types import Number
 
 from df.config import config
 from df.model import ModelParams
 
 try:
-    from torchaudio.functional import resample
+    from torchaudio.functional import resample as ta_resample
 except ImportError:
-    from torchaudio.compliance.kaldi import resample_waveform as resample  # type: ignore
+    from torchaudio.compliance.kaldi import resample_waveform as ta_resample  # type: ignore
 
-resample = resample
+
+def get_resample_params(method: str) -> Dict[str, Any]:
+    params = {
+        "sinc_fast": {"resampling_method": "sinc_interpolation", "lowpass_filter_width": 16},
+        "sinc_best": {"resampling_method": "sinc_interpolation", "lowpass_filter_width": 64},
+        "kaiser_fast": {
+            "resampling_method": "kaiser_window",
+            "lowpass_filter_width": 16,
+            "rolloff": 0.85,
+            "beta": 8.555504641634386,
+        },
+        "kaiser_best": {
+            "resampling_method": "kaiser_window",
+            "lowpass_filter_width": 16,
+            "rolloff": 0.9475937167399596,
+            "beta": 14.769656459379492,
+        },
+    }
+    assert method in params.keys(), f"method must be one of {list(params.keys())}"
+    return params[method]
+
+
+def resample(audio: Tensor, orig_sr: int, new_sr: int, method="sinc_fast"):
+    params = get_resample_params(method)
+    return ta_resample(audio, orig_sr, new_sr, **params)
 
 
 def get_device():
@@ -52,6 +77,36 @@ def as_real(x: Tensor):
     return x
 
 
+class angle_re_im(Function):
+    """Similar to torch.angle but robustify the gradient for zero magnitude."""
+
+    @staticmethod
+    def forward(ctx, re: Tensor, im: Tensor):
+        ctx.save_for_backward(re, im)
+        return torch.atan2(im, re)
+
+    @staticmethod
+    def backward(ctx, grad: Tensor) -> Tuple[Tensor, Tensor]:
+        re, im = ctx.saved_tensors
+        grad_inv = grad / (re.square() + im.square()).clamp_min_(1e-10)
+        return -im * grad_inv, re * grad_inv
+
+
+class angle(Function):
+    """Similar to torch.angle but robustify the gradient for zero magnitude."""
+
+    @staticmethod
+    def forward(ctx, x: Tensor):
+        ctx.save_for_backward(x)
+        return torch.atan2(x.imag, x.real)
+
+    @staticmethod
+    def backward(ctx, grad: Tensor):
+        (x,) = ctx.saved_tensors
+        grad_inv = grad / (x.real.square() + x.imag.square()).clamp_min_(1e-10)
+        return torch.view_as_complex(torch.stack((-x.imag * grad_inv, x.real * grad_inv), dim=-1))
+
+
 def check_finite_module(obj, name="Module", _raise=True) -> Set[str]:
     out: Set[str] = set()
     if isinstance(obj, torch.nn.Module):
@@ -132,7 +187,7 @@ def get_commit_hash():
 
 
 def get_host() -> str:
-    return os.uname().nodename
+    return gethostname()
 
 
 def get_branch_name():
@@ -146,70 +201,6 @@ def get_branch_name():
     return branch
 
 
-def clip_grad_norm_(
-    parameters,
-    max_norm: float,
-    norm_type: float = 2.0,
-    error_if_nonfinite: bool = False,
-) -> torch.Tensor:
-    r"""Pytorch 1.9 backport: Clips gradient norm of an iterable of parameters.
-
-    The norm is computed over all gradients together, as if they were
-    concatenated into a single vector. Gradients are modified in-place.
-
-    Args:
-        parameters (Iterable[Tensor] or Tensor): an iterable of Tensors or a
-            single Tensor that will have gradients normalized
-        max_norm (float or int): max norm of the gradients
-        norm_type (float or int): type of the used p-norm. Can be ``'inf'`` for
-            infinity norm.
-        error_if_nonfinite (bool): if True, an error is thrown if the total
-            norm of the gradients from :attr:``parameters`` is ``nan``,
-            ``inf``, or ``-inf``. Default: False (will switch to True in the future)
-
-    Returns:
-        Total norm of the parameters (viewed as a single vector).
-    """
-    if isinstance(parameters, torch.Tensor):
-        parameters = [parameters]
-    parameters = [p for p in parameters if p.grad is not None]
-    max_norm = float(max_norm)
-    norm_type = float(norm_type)
-    if len(parameters) == 0:
-        return torch.tensor(0.0)
-    device = parameters[0].grad.device
-    if norm_type == torch._six.inf:
-        norms = [p.grad.detach().abs().max().to(device) for p in parameters]
-        total_norm = norms[0] if len(norms) == 1 else torch.max(torch.stack(norms))
-    else:
-        total_norm = torch.norm(
-            torch.stack([torch.norm(p.grad.detach(), norm_type).to(device) for p in parameters]),
-            norm_type,
-        )
-    if total_norm.isnan() or total_norm.isinf():
-        if error_if_nonfinite:
-            raise RuntimeError(
-                f"The total norm of order {norm_type} for gradients from "
-                "`parameters` is non-finite, so it cannot be clipped. To disable "
-                "this error and scale the gradients by the non-finite norm anyway, "
-                "set `error_if_nonfinite=False`"
-            )
-        else:
-            warnings.warn(
-                "Non-finite norm encountered in torch.nn.utils.clip_grad_norm_; continuing anyway. "
-                "Note that the default behavior will change in a future release to error out "
-                "if a non-finite total norm is encountered. At that point, setting "
-                "error_if_nonfinite=false will be required to retain the old behavior.",
-                FutureWarning,
-                stacklevel=2,
-            )
-    clip_coef = max_norm / (total_norm + 1e-6)
-    if clip_coef < 1:
-        for p in parameters:
-            p.grad.detach().mul_(clip_coef.to(p.grad.device))
-    return total_norm
-
-
 # from pytorch/ignite:
 def apply_to_tensor(input_, func):
     """Apply a function on a tensor or mapping, or sequence of tensors."""
